///////////////////////////////////////////////////////////////////////////////
//
// Module: TestApp.cpp
//
// Abstract: This is a test program for cs1030 lab 6 - Personal Organizer.
// Its purpose is to provide a menu-driven test framework for the PO classes.
// Its menu-driven interface allows you to invoke the various functions of 
// your PO.
//
//
///////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <fstream>
#include <string>
#include <cstdio>
#include <sstream>

#include "TestApp.h"
#include "PersonalOrganizer.h"

using namespace std;

//////////////////////////////////////////////////////////////////////////////
// 
// main()
// 
// implements the menu interface for this test application and calls the
// various testing functions based on menu selection
//
/////////////////////////////////////////////////////////////////////////////
int main()
{
	bool repeatMenu = true; // this flag is set false when the user selects the "quit" option
	const int menuMinValue = 0;
	const int menuMaxValue = 11;

	// various menu selections...
	const int menuQuit = 0;				// quit test program
	const int menuEnterCI = 1;			// add a Contact Info item to the Personal Organizer database
	const int menuEnterTask = 2;		// add a Task item to the PO database
	const int menuListCIbyFirst = 3;	// query CI items by first name 
	const int menuListCIbyLast = 4;		// query CI items by last name 
	const int menuListTasksByDate = 5;	// query and list all tasks by date
	const int menuListTasksByDesc = 6;	// query and list tasks by description
	const int menuDeleteTask = 7;		// delete a specific task
	const int menuDeleteCI = 8;			// delete a specific contact
	const int menuSaveDB = 9;			// save database to a file
	const int menuLoadDB = 10;			// load database from that file

	while( repeatMenu )		// keep looping until the user quits
	{
		// post the menu to the user
		system("cls");
		cout << endl <<
			"_________________________Menu_________________________"
			<< endl << endl <<
			"\t 1. Add a Contact Item" << endl <<
			"\t 2. Add a Task Item" << endl <<
			"\t 3. List all Contacts by First Name" << endl <<
			"\t 4. List all Contacts by Last Name" << endl <<
			"\t 5. List all Tasks by Due Date" << endl <<
			"\t 6. List all Tasks by Description" << endl <<
			"\t 7. Delete a Task" << endl <<
			"\t 8. Delete a Contact" << endl <<
			"\t 9. Save to File" << endl <<
			"\t10. Load from File" << endl << endl <<
			"\t 0. Quit" << endl << endl <<
			"Enter a choice: ";

		int menuSelectionValue = 0; // user's menu choice as a numeric value
		bool isEntryValid = false; // indicates whether the user's menu choice was valid
		while( isEntryValid == false ) // keep prompting until a valid selecting is made
		{
			string menuSelectionString; //user's choice entered as a string
			getline (cin, menuSelectionString);	// get the user's entry (could be any arbitrary text string at this point)

			istringstream userInputStream(menuSelectionString); // istringstream used for subsequent parsing of the input string
			userInputStream >> menuSelectionValue; // try to convert the string selection to a numeric value

			// Perform a sanity check on the user's input
			char extraneousInputChar; // used below to see if the user input too many characters for the menu selection

			if (!userInputStream
				|| userInputStream >> extraneousInputChar // other junk in the input string
				|| menuSelectionValue < menuMinValue	// invalid numerical value
				|| menuSelectionValue > menuMaxValue)
			{	// invalid numerical value
				cout << endl << "Invalid Input!  Enter again: ";
			} else {
				isEntryValid = true;	// good input
			}
		}

		// decide what the user selected and perform the associated command
		switch( menuSelectionValue )
		{
		case menuEnterCI:	// Enter a Contact item
			handleEnterCI();
			break;

		case menuEnterTask:	// Enter a Task item
			handleEnterTask();
			break;

		case menuListCIbyFirst:	// List all Contacts by first name
			handleListCI(true);
			break;

		case menuListCIbyLast: // List all Contacts by last name
			handleListCI(false);
			break;

		case menuListTasksByDate:	// List all tasks sorted by due date
			handleListTasks(true);
			break;

		case menuListTasksByDesc: // list all tasks sorted by description
			handleListTasks(false);
			break;

		case menuDeleteTask:	// delete a task
			handleDeleteTask();
			break;

		case menuDeleteCI:		// delete a contact item
			handleDeleteCI();
			break;

		case menuSaveDB:		// output database to a file
			handleSaveDB();
			break;

		case menuLoadDB:		// load database from file
			handleLoadDB();
			break;

		case menuQuit:	// Quit this test application
			repeatMenu = false;
			exit(0);
			break;

		default: // user somehow entered something they shouldn't have
			cout << endl << "Invalid entry; Try again" << endl;
			break;
		}

	}
	return EXIT_SUCCESS;
} 

////////////////////////////////////////////////////////////////////////
// handleEnterCI
// 
// Test the Contact Information entry function of the Personal Organizer
//
// parameters: None;
//
// Returns: void;
//
// Notes: This function prompts the tester for the information to be used 
// to create a Contact Information item and add that item to the PO database; how
// this item is added depends on the design of your system and its interfaces
//////////////////////////////////////////////////////////////////////////
void handleEnterCI(void) 
{
	system("cls");
	string ln, fn, phn;
	cout << "Contact Information Entry Test" << endl << endl;
	cout << "Last Name: ";
	getline(cin,ln);
	cout << endl << "First Name: ";
	getline(cin,fn);
	cout << endl << "Phone Number (000-000-0000): ";
	getline(cin,phn);

	// CS1030 TODO: create a new ContactItem here and add it to the collection

	cout << endl << endl << "Contact created successfully." << endl << endl;

	system("pause");
}

////////////////////////////////////////////////////////////////////////
// handleEnterTask
// 
// Test the Task entry function of the Personal Organizer
//
// parameters: None;
//
// Returns: void;
//
// Notes: This function prompts the tester for the information to be used 
// to create a Task item and add that item to the PO database; how
// this item is added depends on the design of your system and its interfaces
//////////////////////////////////////////////////////////////////////////
void handleEnterTask(void)
{
	system("cls");
	string strMonth, strDay, strYear, strDesc; // string values read from cin
	int iMonth, iDay, iYear;				   // equivalent numeric values
	cout << "Task Information Entry Test" <<
		endl << endl << "Enter a brief description: ";
	getline(cin, strDesc);		// obtain the description
	cout << endl << endl << "Due Date: " << endl <<
		"\tMonth: ";
	getline(cin, strMonth);	// get date parameters
	cout << "\tDay: ";
	getline(cin, strDay);
	cout << "\tYear: ";
	getline(cin, strYear);	// get and cast strings into integers
	istringstream(strMonth) >> iMonth;
	istringstream(strDay) >> iDay;
	istringstream(strYear) >> iYear;

	// CS1030 TODO: create a new TaskItem here and add it to the collection

	cout << endl << endl <<
		"Task item created successfully." <<
		endl << endl;
	system("pause");
}

////////////////////////////////////////////////////////////////////////
// handleListCI
// 
// Test the List All Contacts function of the Personal Organizer
//
// paramters: sortMethod - determines sort method: 
//				true by first name; false by last
//
// Returns: void
//
// Notes: This function should query the PO database for Contacts and
// list them to the console in a formatted fashion.
//////////////////////////////////////////////////////////////////////////
void handleListCI(bool sortMethod)
{
	system("cls");
	// CS1030 TODO: complete this method

	system("pause");
}


////////////////////////////////////////////////////////////////////////
// handleListTasks
// 
// Test the List All Tasks function of the Personal Organizer
//
// paramters: sortMethod - determines sort method:
//				true by due Date; false by Description
//
// Returns: void
//
// Notes: This function should query the PO database for Tasks and
// list them to the console in a formatted fashion.
//////////////////////////////////////////////////////////////////////////
void handleListTasks(bool sortMethod)
{
	system("cls");
	// CS1030 TODO: complete this method

	system("pause");
}

////////////////////////////////////////////////////////////////////////
// handleDeleteTask
// 
// Test the Delete Task function of the Personal Organizer
//
// paramters: None
//
// Returns: void
//
// Notes: This function should query the PO database for a task with a certain
// ID and then remove it from the collection.
//////////////////////////////////////////////////////////////////////////
void handleDeleteTask(void) {
	system("cls");
	cout << "Delete Task..." <<
		endl << endl << "Enter Task ID #: ";
	string taskIdStr;
	getline(cin, taskIdStr);		// get ID to delete

	// CS1030 TODO: complete this method
	cout << endl << endl << "Task deleted successfully." << endl << endl;

	system("pause");
}

////////////////////////////////////////////////////////////////////////
// handleDeleteContact
// 
// Test the Delete Contact function of the Personal Organizer
//
// paramters: None
//
// Returns: void
//
// Notes: This function should query the PO database for a contact item 
// with a certain ID and then remove it from the collection.
//////////////////////////////////////////////////////////////////////////
void handleDeleteCI(void) {
	system("cls");
	cout << "Delete Contact..." << endl << endl;
	cout << "Enter Contact ID #: ";
	string contIdStr;
	getline(cin, contIdStr);			// get ID to delete
	
	// CS1030 TODO: complete this method
	cout << endl << endl << "Contact deleted successfully." << endl << endl;

	system("pause");
}

////////////////////////////////////////////////////////////////////////
// handleSaveDB
// 
// Test the Save to File feature of the Personal Organizer
//
// paramters: None;
//
// Returns: void;
//
// Notes: This function outputs the contents of the vectors to a text
// file that can be easily reopened and used within the Personal Organizer
//////////////////////////////////////////////////////////////////////////
void handleSaveDB(void) {
	string saveLoc;
	system("cls");
	cout << "Saving Database to File..." << endl << endl;
	cout << "Enter filename: ";
	getline(cin, saveLoc);
	
	// CS1030 TODO: complete this method
	cout << endl << endl << "File saved successfully." << endl << endl;

	system("pause");
}

////////////////////////////////////////////////////////////////////////
// handleLoadDB
// 
// Test the Load from File feature of the Personal Organizer
//
// paramters: None
//
// Returns: void
//
// Notes: This function parses the contents of text file and turns it back
// into Task and Contact items
//////////////////////////////////////////////////////////////////////////
void handleLoadDB(void) {
	string loadLoc;
	system("cls");
	cout << "Loading database from File..." << endl << endl;
	cout << "Enter filename: ";
	getline(cin, loadLoc);

	// CS1030 TODO: complete this method
	cout << endl << endl << "File loaded successfully." << endl << endl;

	system("pause");
}
